from google.protobuf.json_format import MessageToDict
from .proto import AttestationSpecification, ComputeNodeProtocol, DriverTaskConfig
from typing import List, Dict, Optional, Any
from typing_extensions import TypedDict
from enum import Enum
from .proto.length_delimited import parse_length_delimited
from .proto.compute_sql_pb2 import SqlWorkerConfiguration
from .container.proto.compute_container_pb2 import ContainerWorkerConfiguration


__all__ = [
    "EnclaveSpecification",
    "JobId",
    "DataRoomDescription",
    "DatasetDescription",
]


class JobId:
    """
    Class for identifying running or already run jobs.

    Objects of this class can be used to retrieve results for processed computations.
    """
    def __init__(self, job_id: str, compute_node_id: str):
        self.id = job_id
        """The identifier of the job that processed a particular computation."""

        self.compute_node_id = compute_node_id
        """The id of the computation that was processed."""


class ScopeTypes(str, Enum):
    DATASET = "DATASET",
    DCR_DATA = "DCR_DATA"


class UserResponse(TypedDict):
    id: str
    email: str


class UserCsrRequest(TypedDict):
    csrPem: str


class UserCsrResponse(TypedDict):
    certChainPem: str


class SystemCaResponse(TypedDict):
    rootCertificate: str


class CreateSessionRequest(TypedDict):
    attestationSpecificationHash: str


class SessionJsonResponse(TypedDict):
    sessionId: str
    attestationSpecificationHash: str


class FinalizeUpload(TypedDict):
    uploadId: str
    manifest: str
    name: str
    manifestHash: str
    chunks: List[str]
    scopeId: str


class ChunkWrapper(TypedDict):
    hash: str
    data: str


class UploadDescription(TypedDict):
    uploadId: str


class ChunkDescription(TypedDict):
    chunkHash: str


class UserDescription(TypedDict):
    id: str
    email: str


class DataRoomDescription(TypedDict):
    """The identifier of the data room"""
    id: str
    """The title that was given to the data room"""
    title: str
    """The hex-encoded hash of the driver attestation specification"""
    driverAttestationHash: str
    """Whether the data room has been stopped"""
    isStopped: bool
    """When the data room was created"""
    createdAt: str
    """When the data room was last updated"""
    updatedAt: str


class DatasetDescription(TypedDict):
    """
    This class includes information about an uploaded dataset
    """
    id: str
    """The identifier of this dataset"""
    manifestHash: str
    """The data set id as a hex-encoded string. This id is also called the manifest hash."""
    name: str
    """The name of this dataset"""
    description: str
    """An optional description"""
    createdAt: str
    """When the dataset was uploaded"""


class SignatureResponse(TypedDict):
    type: str
    data: List[int]


class FatquoteResBody(TypedDict):
    fatquoteBase64: str


class DatasetManifestMetadata(TypedDict):
    name: str
    manifestHash: str
    chunks: List[str]


class EnclaveSpecification(TypedDict):
    """
    This class includes information about an enclave deployed in the platform.
    Please refer to `decentriq_platform.EnclaveSpecifications` for a detailed explanation.
    """
    proto: AttestationSpecification
    """The Protobuf object."""
    workerProtocols: List[int]
    """The worker protocol versions supported by the node"""
    decoder: Optional[Any]
    """
    Decoder object that can be used to decode the binary configs belonging
    to enclaves of this type.
    """
    clientProtocols: Optional[List[int]]
    """The client protocol versions supported by the node"""


class EnclaveSpecificationJson(TypedDict):
    name: str
    version: str
    spec: str


class EnclaveSpecificationResponse(TypedDict):
    attestationSpecs: List[EnclaveSpecificationJson]


class Tcb(TypedDict):
    sgxtcbcomp01svn: int
    sgxtcbcomp02svn: int
    sgxtcbcomp03svn: int
    sgxtcbcomp04svn: int
    sgxtcbcomp05svn: int
    sgxtcbcomp06svn: int
    sgxtcbcomp07svn: int
    sgxtcbcomp08svn: int
    sgxtcbcomp09svn: int
    sgxtcbcomp10svn: int
    sgxtcbcomp11svn: int
    sgxtcbcomp12svn: int
    sgxtcbcomp13svn: int
    sgxtcbcomp14svn: int
    sgxtcbcomp15svn: int
    sgxtcbcomp16svn: int
    pcesvn: int


class TcbLevel(TypedDict):
    tcb: Tcb
    tcbStatus: str


class TcbInfo(TypedDict):
    version: int
    issueDate: str
    nextUpdate: str
    fmspc: str
    pceId: str
    tcbType: int
    tcbEvaluationDataNumber: int
    tcbLevels: List[TcbLevel]


class TcbInfoContainer(TypedDict):
    tcbInfo: TcbInfo
    signature: str


class IasResponse(TypedDict):
    isvEnclaveQuoteBody: str
    isvEnclaveQuoteStatus: str


class KeychainInstance(TypedDict):
    userId: str
    salt: str
    encrypted: bytes
    casIndex: int


"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class S3Source(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUCKET_FIELD_NUMBER: builtins.int
    REGION_FIELD_NUMBER: builtins.int
    OBJECTKEY_FIELD_NUMBER: builtins.int
    bucket: builtins.str
    region: builtins.str
    objectKey: builtins.str
    def __init__(
        self,
        *,
        bucket: builtins.str = ...,
        region: builtins.str = ...,
        objectKey: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["bucket", b"bucket", "objectKey", b"objectKey", "region", b"region"]) -> None: ...

global___S3Source = S3Source

class DataSourceS3WorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOURCE_FIELD_NUMBER: builtins.int
    CREDENTIALSDEPENDENCY_FIELD_NUMBER: builtins.int
    @property
    def source(self) -> global___S3Source: ...
    credentialsDependency: builtins.str
    def __init__(
        self,
        *,
        source: global___S3Source | None = ...,
        credentialsDependency: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["source", b"source"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["credentialsDependency", b"credentialsDependency", "source", b"source"]) -> None: ...

global___DataSourceS3WorkerConfiguration = DataSourceS3WorkerConfiguration



from __future__ import annotations
import typing
from typing import Dict, Literal, Optional, Union, Tuple, List
from .client import Client, KeychainInstance
import cbor2
import chily
from base64 import b64encode

KeychainEntryKind = Literal["dataset_key", "other_secret"]

class KeychainEntry:
    kind: KeychainEntryKind
    key: str
    value: bytes

    def __init__(self, kind: KeychainEntryKind, key: str, value: bytes):
        self.kind = kind
        self.key = key
        self.value = value

class KeychainDecryptException(Exception):
    def __init__(self) -> None:
        super().__init__()    

class Keychain:

    _client: Client
    _store: Dict[str, bytes]
    _secret_wrapper: chily.SecretWrapper
    _keychain_instance: KeychainInstance

    def __init__(
            self,
            client: Client,
            secret_wrapper: chily.SecretWrapper,
            keychain_instance: KeychainInstance,
            store: Dict[str, bytes],
    ):
        self._client = client
        self._store = store
        self._secret_wrapper = secret_wrapper
        self._keychain_instance = keychain_instance
    
    @staticmethod
    def _serialize_store(store: Dict[str, bytes]) -> bytes:
        return cbor2.dumps(store, value_sharing=False)
    
    @staticmethod
    def _deserialize_store(plaintext: bytes) -> Dict[str, bytes]:
        return cbor2.loads(plaintext)
    
    def _namespaced_key(self, kind: KeychainEntryKind, key: str) -> str:
        return f"{kind}/{key}"
    
    @staticmethod
    def _parse_namespaced_key(ns_key: str) -> Tuple[KeychainEntryKind, str]:
        splitted = ns_key.split("/")
        if len(splitted) < 2:
            raise Exception("Invalid namespaced key: no `/` found")
        if splitted[0] not in typing.get_args(KeychainEntryKind):
            raise Exception(f"Invalid namespaced key kind: found {splitted[0]}")
        remainder = "/".join(splitted[1:])
        return [splitted[0], remainder]

    @staticmethod    
    def _encrypt_store_static(
            user_email: str,
            secret_wrapper: chily.SecretWrapper,
            store: Dict[str, bytes]
    ) -> bytes:
        return secret_wrapper.wrap_secret(user_email, Keychain._serialize_store(store))

    def _encrypt_store(
            self,
    ):
        self._keychain_instance["encrypted"] = Keychain._encrypt_store_static(
            self._client.user_email,
            self._secret_wrapper,
            self._store
        )

    @staticmethod
    def _decrypt_store_static(
            user_email: str,
            secret_wrapper: chily.SecretWrapper,
            encrypted: bytes
    ) -> Dict[str, bytes]:
        try:
            decrypted = secret_wrapper.unwrap_secret(user_email, encrypted)
        except:
            raise KeychainDecryptException()
        return Keychain._deserialize_store(decrypted)

    def _decrypt_store(self):
        self._store = Keychain._decrypt_store_static(
            self._client.user_email,
            self._secret_wrapper,
            self._keychain_instance["encrypted"]
        )

    def _set_keychain_instance(self, keychain_instance: KeychainInstance):
        self._keychain_instance = keychain_instance
        self._decrypt_store()

    def _set_store(self, store: Dict[str, bytes]):
        self._store = store
        self._encrypt_store()

    def _init_keychain_instance(client: Client) -> KeychainInstance:
        keychain_instance = client.get_keychain_instance()
        if not keychain_instance:
            raise Exception("Cannot init Keychain, no instance exists in the server, create a new instance first")
        return keychain_instance
    
    def _has_keychain_instance(client: Client) -> bool:
        keychain_instance = client.get_keychain_instance()
        return bool(keychain_instance)
    
    def _create_new_keychain_with_secret_wrapper(
            client: Client,
            secret_wrapper: chily.SecretWrapper
    ) -> Optional[Keychain]:
        if (Keychain._has_keychain_instance(client)):
            return None
        store: Dict[str, bytes] = {}
        encrypted = secret_wrapper.wrap_secret(client.user_email, Keychain._serialize_store(store))
        keychain_instance = client.create_keychain_instance(secret_wrapper.salt, encrypted)
        return Keychain(client, secret_wrapper, keychain_instance, store)
    
    def create_new_keychain(client: Client, password: bytes) -> Keychain:
        secret_wrapper = chily.SecretWrapper.init(password)
        return Keychain._create_new_keychain_with_secret_wrapper(client, secret_wrapper)
    
    def create_new_keychain_with_master_key(
            client: Client,
            master_key: bytes,
            salt: str,
    ) -> Optional[Keychain]:
        secret_wrapper = chily.SecretWrapper.with_master_key(master_key, salt)
        return Keychain._create_new_keychain_with_secret_wrapper(client, secret_wrapper)

    def init_with_master_key(client: Client, master_key: bytes) -> Keychain:
        keychain_instance = Keychain._init_keychain_instance(client)
        secret_wrapper = chily.SecretWrapper.with_master_key(master_key, keychain_instance["salt"])
        store = Keychain._decrypt_store_static(client.user_email, secret_wrapper, keychain_instance["encrypted"])
        return Keychain(client, secret_wrapper, keychain_instance, store)

    def init_with_password(client: Client, password: bytes) -> Keychain:
        keychain_instance = Keychain._init_keychain_instance(client)
        secret_wrapper = chily.SecretWrapper.with_password(password, keychain_instance["salt"])
        store = Keychain._decrypt_store_static(client.user_email, secret_wrapper, keychain_instance["encrypted"])
        return Keychain(client, secret_wrapper, keychain_instance, store)
    
    def _download(self):
        keychain_instance = self._client.get_keychain_instance()
        if not keychain_instance:
            raise Exception("Keychain instance has not been created yet")
        
        if self._secret_wrapper.salt != keychain_instance["salt"]:
            raise Exception("Keychain salt has changed upstream. Reinitialize Keychain with password first")
        
        if keychain_instance["casIndex"] != self._keychain_instance["casIndex"]:
            self._set_keychain_instance(keychain_instance)
        return
        
    def get(self, kind: KeychainEntryKind, key: str) -> Optional[KeychainEntry]:
        self._download()
        ns_key = self._namespaced_key(kind, key)
        value = self._store.get(ns_key)
        if value:
            return KeychainEntry(kind, key, value)
        else:
            return None
        
    def items(self) -> List[KeychainEntry]:
        self._download()
        items = []
        for ns_key, value in self._store.items():
            kind, key = Keychain._parse_namespaced_key(ns_key)
            items.append(KeychainEntry(kind, key, value))
        return items
    
    def _insert_local(self, entry: KeychainEntry):
        ns_key = self._namespaced_key(entry.kind, entry.key)
        if self._store.get(ns_key):
            raise Exception("Cannot insert new entry: an entry already exists for this (kind, key) pair")
        if entry.kind in ["dataset_key", "other_secret"]:
            self._store[ns_key] = entry.value
        else:
            raise Exception(f"Invalid entry kind: {entry.kind}")
        self._encrypt_store()
        return

    def _compare_and_swap(self) -> bool:
        return self._client.compare_and_swap_keychain(
            self._keychain_instance["casIndex"],
            self._keychain_instance["salt"],
            self._keychain_instance["encrypted"],
        )
    
    def insert(self, entry: KeychainEntry):
        while True:
            self._insert_local(entry)
            if self._compare_and_swap():
                return
            self._download()
    
    def remove(self, kind: KeychainEntryKind, key: str):
        ns_key = self._namespaced_key(kind, key)
        while True:
            if self._store.pop(ns_key):
                self._encrypt_store()
                if self._compare_and_swap():
                    return
                self._download()
            else:
                raise Exception("Cannot delete entry: Entry does not exist")
            
    def reset(client: Client):
        return client.reset_keychain()
    
    def clear(self):
        while True:
            self._set_store({})
            if self._compare_and_swap():
                return
            self._download()

    def change_password(self, new_password: bytes):
        new_secret_wrapper = chily.SecretWrapper.with_password(new_password, self._secret_wrapper.salt)
        return self.change_master_key(new_secret_wrapper.master_key)
    
    def change_master_key(self, new_master_key: bytes):
        new_secret_wrapper = chily.SecretWrapper.with_master_key(new_master_key, self._secret_wrapper.salt)
        while True:
            new_keychain = Keychain(self._client, new_secret_wrapper, self._keychain_instance, self._store)
            new_keychain._encrypt_store()
            if new_keychain._compare_and_swap():
                self = new_keychain
                return
            self._download()

    def get_master_key(self) -> bytes:
        return self._secret_wrapper.master_key


from google.protobuf.json_format import MessageToDict
from ..proto import serialize_length_delimited, ComputeNodeFormat, parse_length_delimited
from ..node import Node
from typing import Optional
from .proto import (
    DatasetSinkWorkerConfiguration,
)

__docformat__ = "restructuredtext"
__pdoc__ = {
    "proto": False,
}

class DatasetSink(Node):
    """
    Compute node that re-encrypts input data with a user-provided key
    and creates a dataset entry in the database.
    """

    def __init__(
            self,
            name: str,
            input_dependency: str,
            encryption_key_dependency: str,
            dataset_name: str,
            dataset_scope_id,
            dataset_description: Optional[str],
            dataset_import_id: Optional[str] = None,
    ) -> None:
        config = DatasetSinkWorkerConfiguration(
            inputDependency=input_dependency,
            encryptionKeyDependency=encryption_key_dependency,
            datasetName=dataset_name,
            datasetDescription=dataset_description,
            datasetScopeId=dataset_scope_id,
            datasetImportId=dataset_import_id,
        )
        config_serialized = serialize_length_delimited(config)
        super().__init__(
            name,
            config=config_serialized,
            enclave_type="decentriq.dataset-sink-worker",
            dependencies=[input_dependency, encryption_key_dependency],
            output_format=ComputeNodeFormat.ZIP
        )


class DatasetSinkWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = DatasetSinkWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)

__all__ = [
    "DatasetSink",
]



from .data_room_pb2 import (
    Permission,
    LeafCrudPermission,
    RetrieveAuditLogPermission,
    ExecuteComputePermission,
    RetrieveComputeResultPermission,
    RetrieveDataRoomPermission,
    RetrieveDataRoomStatusPermission,
    GenerateMergeSignaturePermission,
    UpdateDataRoomStatusPermission,
    RetrievePublishedDatasetsPermission,
    ExecuteDevelopmentComputePermission,
    MergeConfigurationCommitPermission,
    DryRunPermission,
    AuthenticationMethod, UserPermission, ComputeNode,
    ComputeNodeLeaf, ComputeNodeBranch, DataRoom,
    ComputeNodeFormat,
    ComputeNodeProtocol,
    ConfigurationModification,
    AddModification,
    ChangeModification,
    ConfigurationElement,
    GovernanceProtocol,
    StaticDataRoomPolicy,
    AffectedDataOwnersApprovePolicy,
    ConfigurationCommit,
    DataRoomConfiguration,
    PkiPolicy,
    DcrSecretPolicy,
)
from .attestation_pb2 import (
    AttestationSpecification,
    AttestationSpecificationIntelEpid,
    Fatquote,
    FatquoteEpid, FatquoteDcap,
    AttestationSpecificationIntelDcap,
    AttestationSpecificationAwsNitro,
    AttestationSpecificationAmdSnp,
)
from .gcg_pb2 import (
    DriverTaskConfig, NoopConfig, DataRoomStatus,
    CreateDataRoomRequest, CreateDataRoomResponse,
    ExecuteComputeRequest, ExecuteComputeResponse, GcgRequest, GcgResponse, GetResultsRequest,
    GetResultsResponseChunk, JobStatusRequest, JobStatusResponse,
    PublishDatasetToDataRoomRequest, PublishDatasetToDataRoomResponse,
    RemovePublishedDatasetRequest, RemovePublishedDatasetResponse,
    RetrieveAuditLogRequest, RetrieveAuditLogResponse, RetrieveDataRoomRequest,
    RetrieveDataRoomResponse, RetrieveDataRoomStatusRequest,
    RetrieveDataRoomStatusResponse, RetrievePublishedDatasetsRequest,
    RetrievePublishedDatasetsResponse, UpdateDataRoomStatusRequest,
    ExecuteDevelopmentComputeRequest, CreateConfigurationCommitRequest,
    GenerateMergeApprovalSignatureRequest, MergeConfigurationCommitRequest,
    CreateConfigurationCommitResponse, GenerateMergeApprovalSignatureResponse,
    MergeConfigurationCommitResponse, UpdateDataRoomStatusResponse,
    RetrieveCurrentDataRoomConfigurationRequest, RetrieveCurrentDataRoomConfigurationResponse,
    RetrieveConfigurationCommitRequest, RetrieveConfigurationCommitResponse, StaticContentConfig,
    CreateConfigurationCommitRequest, RetrieveConfigurationCommitApproversRequest,
    UserAuth, Pki
)
from .length_delimited import parse_length_delimited, serialize_length_delimited
from .delta_enclave_api_pb2 import (
    ChunkHeader, EncryptionHeader, VersionHeader,
    DataNoncePubkey, Request, Response
)
from .synth_data_pb2 import (
    SyntheticDataConf,
    Mask,
    Column,
)
from .metering_pb2 import (
    DcrMetadata,
    CreateDcrPurpose,
    CreateDcrKind
)
from .identity_endorsement_pb2 import (
    DcrSecretEndorsementRequest,
    DcrSecretEndorsementResponse,
    EndorsementRequest,
    EndorsementResponse,
    PkiEndorsementRequest,
    PkiEndorsementResponse,
    EnclaveEndorsement,
    EnclaveEndorsements,
)




"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ContainerWorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STATIC_FIELD_NUMBER: builtins.int
    @property
    def static(self) -> global___StaticImage: ...
    def __init__(
        self,
        *,
        static: global___StaticImage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["configuration", b"configuration", "static", b"static"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["configuration", b"configuration", "static", b"static"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["configuration", b"configuration"]) -> typing_extensions.Literal["static"] | None: ...

global___ContainerWorkerConfiguration = ContainerWorkerConfiguration

class StaticImage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMAND_FIELD_NUMBER: builtins.int
    MOUNTPOINTS_FIELD_NUMBER: builtins.int
    OUTPUTPATH_FIELD_NUMBER: builtins.int
    INCLUDECONTAINERLOGSONERROR_FIELD_NUMBER: builtins.int
    INCLUDECONTAINERLOGSONSUCCESS_FIELD_NUMBER: builtins.int
    @property
    def command(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def mountPoints(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MountPoint]: ...
    outputPath: builtins.str
    includeContainerLogsOnError: builtins.bool
    includeContainerLogsOnSuccess: builtins.bool
    def __init__(
        self,
        *,
        command: collections.abc.Iterable[builtins.str] | None = ...,
        mountPoints: collections.abc.Iterable[global___MountPoint] | None = ...,
        outputPath: builtins.str = ...,
        includeContainerLogsOnError: builtins.bool = ...,
        includeContainerLogsOnSuccess: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["command", b"command", "includeContainerLogsOnError", b"includeContainerLogsOnError", "includeContainerLogsOnSuccess", b"includeContainerLogsOnSuccess", "mountPoints", b"mountPoints", "outputPath", b"outputPath"]) -> None: ...

global___StaticImage = StaticImage

class MountPoint(google.protobuf.message.Message):
    """Dependencies are mounted under the `/input` directory
    For example for a mount point entry { path: "/data", dependency: "dep" }
    the worker will mount the dependency `dep` at path `/input/data`
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PATH_FIELD_NUMBER: builtins.int
    DEPENDENCY_FIELD_NUMBER: builtins.int
    path: builtins.str
    dependency: builtins.str
    def __init__(
        self,
        *,
        path: builtins.str = ...,
        dependency: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dependency", b"dependency", "path", b"path"]) -> None: ...

global___MountPoint = MountPoint



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dataset_sink.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x64\x61taset_sink.proto\x12\x0c\x64\x61taset_sink\"\xf1\x01\n\x1e\x44\x61tasetSinkWorkerConfiguration\x12\x17\n\x0finputDependency\x18\x01 \x01(\t\x12\x1f\n\x17\x65ncryptionKeyDependency\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x61tasetName\x18\x03 \x01(\t\x12\x16\n\x0e\x64\x61tasetScopeId\x18\x04 \x01(\t\x12\x1f\n\x12\x64\x61tasetDescription\x18\x05 \x01(\tH\x00\x88\x01\x01\x12\x1c\n\x0f\x64\x61tasetImportId\x18\x06 \x01(\tH\x01\x88\x01\x01\x42\x15\n\x13_datasetDescriptionB\x12\n\x10_datasetImportIdb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dataset_sink_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DATASETSINKWORKERCONFIGURATION._serialized_start=37
  _DATASETSINKWORKERCONFIGURATION._serialized_end=278
# @@protoc_insertion_point(module_scope)



from .proto import AttestationSpecification, DataRoomConfiguration
from google.protobuf.json_format import MessageToDict
from .types import EnclaveSpecification
from typing import Dict, List


def decode_compute_node_config(
        config : bytes,
        attestation_spec: AttestationSpecification,
        enclave_specs: List[EnclaveSpecification],
):
    for enclave_spec in enclave_specs:
        if enclave_spec["proto"] == attestation_spec:
            decoder = enclave_spec["decoder"]
            if decoder:
                config_decoded = decoder.decode(config)
                return config_decoded
    return None


def decode_compute_node_configs(
        data_room_configuration: DataRoomConfiguration,
        enclave_specs: List[EnclaveSpecification]
) -> Dict[str, Dict]:
    current_configuration = {}
    for element in data_room_configuration.elements:
        current_configuration[element.id] = element
    decoded_configuration = {}

    for element in data_room_configuration.elements:
        if element.HasField("computeNode"):
            if element.computeNode.HasField("branch"):
                config = element.computeNode.branch.config
                index = element.computeNode.branch.attestationSpecificationId
                attestation_spec = current_configuration[index].attestationSpecification
                config_decoded = decode_compute_node_config(config, attestation_spec, enclave_specs)
                if config_decoded:
                    decoded_configuration[element.id] = config_decoded

    return decoded_configuration


def decode_data_room_configuration(
        data_room_configuration: DataRoomConfiguration,
        enclave_specifications: List[EnclaveSpecification],
):
    decoded_configs = decode_compute_node_configs(data_room_configuration, enclave_specifications)
    decoded_configuration = {}

    for element in data_room_configuration.elements:
        element_json = MessageToDict(element)
        is_branch = "computeNode" in element_json and "branch" in element_json["computeNode"]
        if is_branch and element.id in decoded_configs:
            decoded_config = decoded_configs[element.id]
            if decoded_config:
                element_json["computeNode"]["branch"]["config"] =  decoded_config
        decoded_configuration[element.id] = element_json

    return decoded_configuration



import base64
from typing import Dict, List, Tuple
from .types import EnclaveSpecification
from .compute import GcgDriverDecoder
from .sql.compute import SqlWorkerDecoder
from .container.compute import ContainerWorkerDecoder
from .s3_sink.compute import S3SinkWorkerDecoder
from .dataset_sink import DatasetSinkWorkerDecoder
from .data_source_s3 import DataSourceS3WorkerDecoder
from .proto import (
    AttestationSpecification,
    AttestationSpecificationIntelDcap,
    AttestationSpecificationAwsNitro,
    AttestationSpecificationAmdSnp,
)
import asn1crypto.pem
from .certs import (
    aws_nitro_root_ca_pem,
    amd_snp_ark_pem,
    intel_sgx_dcap_root_ca,
    intel_sgx_ias_root_ca,
)


intel_sgx_dcap_root_ca_der = asn1crypto.pem.unarmor(intel_sgx_dcap_root_ca)[2]
intel_sgx_ias_root_ca_der = asn1crypto.pem.unarmor(intel_sgx_ias_root_ca)[2]
aws_nitro_root_ca_der = asn1crypto.pem.unarmor(aws_nitro_root_ca_pem)[2]
amd_snp_ark_der = asn1crypto.pem.unarmor(amd_snp_ark_pem)[2]


# From https://developers.cloudflare.com/time-services/roughtime/recipes/
roughtime_public_key = base64.b64decode("gD63hSj3ScS+wuOeGrubXlq35N1c5Lby/S+T7MNTjxo=")


# Allowed Decentriq AMD CPU Chip IDs
# This list is hashed and used for the attestation spec, so this list needs to be in the same order in these files:
# - avato-backend/frontend/client/src/attestation.ts
# - trusted/delta-attestation/src/snp.rs
amd_snp_authorized_chip_ids = [
  bytes.fromhex("372499d1b652b98aa1cde6d146136ad2b3e8f93b8223c367087577a73b76813928ad25545327032f7dc44a288965eb8e7f16179dbdb3a71fddc36cc19478ce47"),
  bytes.fromhex("ce8a61aea3f76bbdd05706bcbfb4ade4c65b33ad41f49d0e89dec177951117d247781a3195de2e85399b7117d9eee2f6f2c2e5d21064a4d7e2815380212f0937"),
  bytes.fromhex("86185338a275af6b5e26e06a21b4e0c65db0fc9033e4a24e27cb528321726dc152f28b08c05493c48e8fba3047aba6f0a0dd01d3eebb055b3318c1029c0a74ee"),
  bytes.fromhex("7d2e30af8f43cc6de4e6c3ee59a7ab0d9ddfbff4caba3de0e54430215bd640c571d0e4b49d0cd1502fa74ae8a59475fe06dd9e7635500d584478e13f1191dc27"),
  bytes.fromhex("02926acb4dab55b176946d75e5955154b11269dd8bfe7ff7bcd162b26ea47d8b9557bebd927e90667e34e6dbf9f1bb3bff4bdd36f41d2b755cccabf50270e324"),
]


SPECIFICATIONS = {
    "decentriq.driver:v10": EnclaveSpecification(
        proto=AttestationSpecification(
            intelDcap=AttestationSpecificationIntelDcap(
                mrenclave=bytes.fromhex(
                    "799044e44e189338553b18706e32284725300926569bbec19af576557abfba19"
                ),
                dcapRootCaDer=intel_sgx_dcap_root_ca_der,
                accept_debug=False,
                accept_out_of_date=False,
                accept_configuration_needed=False,
                accept_revoked=False,
            )
        ),
        workerProtocols=[1],
        decoder=GcgDriverDecoder(),
        clientProtocols=[3],
    ),
    "decentriq.driver:v11": EnclaveSpecification(
        proto=AttestationSpecification(
            intelDcap=AttestationSpecificationIntelDcap(
                mrenclave=bytes.fromhex(
                    "661cb988c03197f60a619fc4b6e28980790fefb6ab710f9d8994d42e9d00d8ed"
                ),
                dcapRootCaDer=intel_sgx_dcap_root_ca_der,
                accept_debug=False,
                accept_out_of_date=False,
                accept_configuration_needed=False,
                accept_revoked=False,
            )
        ),
        workerProtocols=[1],
        decoder=GcgDriverDecoder(),
        clientProtocols=[4],
    ),
    "decentriq.driver:v12": EnclaveSpecification(
        proto=AttestationSpecification(
            intelDcap=AttestationSpecificationIntelDcap(
                mrenclave=bytes.fromhex(
                    "b24067659124bebeb3e83d15733a50ad72669a162484c3bb5488dba8e743a1d7"
                ),
                dcapRootCaDer=intel_sgx_dcap_root_ca_der,
                accept_debug=False,
                accept_out_of_date=False,
                accept_configuration_needed=False,
                accept_revoked=False,
            )
        ),
        workerProtocols=[1],
        decoder=GcgDriverDecoder(),
        clientProtocols=[4],
    ),
    "decentriq.sql-worker:v10": EnclaveSpecification(
        proto=AttestationSpecification(
            intelDcap=AttestationSpecificationIntelDcap(
                mrenclave=bytes.fromhex(
                    "dcc9847948837a5cdb85d4fb13d6b77ff6ff5dab63bef35c95901adfa7f1a102"
                ),
                dcapRootCaDer=intel_sgx_dcap_root_ca_der,
                accept_debug=False,
                accept_out_of_date=False,
                accept_configuration_needed=False,
                accept_revoked=False,
            )
        ),
        workerProtocols=[1],
        decoder=SqlWorkerDecoder(),
        clientProtocols=None,
    ),
    "decentriq.python-ml-worker-32-64:v10": EnclaveSpecification(
        proto=AttestationSpecification(
            amdSnp=AttestationSpecificationAmdSnp(
                amdArkDer=amd_snp_ark_der,
                measurement=bytes.fromhex(
                    "9a6a0fd3d0652eae039011346c80cedf572c8e725cbb294a2067ce0a66e7c128c7baf370a77b1005eba3f98f467c9cde"
                ),
                roughtimePubKey=roughtime_public_key,
                authorizedChipIds=amd_snp_authorized_chip_ids,
            )
        ),
        workerProtocols=[1],
        decoder=ContainerWorkerDecoder(),
        clientProtocols=None,
    ),
    "decentriq.python-ml-worker-32-64:v11": EnclaveSpecification(
        proto=AttestationSpecification(
            amdSnp=AttestationSpecificationAmdSnp(
                amdArkDer=amd_snp_ark_der,
                measurement=bytes.fromhex(
                    "683b9ad485cfd3eb25c5d49739d1392552f044e736a1c15a3039407edfb0c3bbd9e397dd27ab866c0417e6acdc0794ca"
                ),
                roughtimePubKey=roughtime_public_key,
                authorizedChipIds=amd_snp_authorized_chip_ids,
            )
        ),
        workerProtocols=[1],
        decoder=ContainerWorkerDecoder(),
        clientProtocols=None,
    ),
    "decentriq.python-ml-worker-32-64:v12": EnclaveSpecification(
        proto=AttestationSpecification(
            amdSnp=AttestationSpecificationAmdSnp(
                amdArkDer=amd_snp_ark_der,
                measurement=bytes.fromhex(
                    "8c17b219d1c0e7ce224b9fd8716bca389f7e1c15df3d791ab7d8ef0dbb85554ae7fa3e7328d5d30cf277bedfd266ee5a"
                ),
                roughtimePubKey=roughtime_public_key,
                authorizedChipIds=amd_snp_authorized_chip_ids,
            )
        ),
        workerProtocols=[1],
        decoder=ContainerWorkerDecoder(),
        clientProtocols=None,
    ),
    "decentriq.python-synth-data-worker-32-64:v10": EnclaveSpecification(
        proto=AttestationSpecification(
            amdSnp=AttestationSpecificationAmdSnp(
                amdArkDer=amd_snp_ark_der,
                measurement=bytes.fromhex(
                    "8b9f780b6524418f4fc8d4bc8b2450c82e09aefb17b36aebfcf0630b6dea8b1451f12c35203d6ddc9c94b74351db9a22"
                ),
                roughtimePubKey=roughtime_public_key,
                authorizedChipIds=amd_snp_authorized_chip_ids,
            )
        ),
        workerProtocols=[1],
        decoder=ContainerWorkerDecoder(),
        clientProtocols=None,
    ),
    "decentriq.python-synth-data-worker-32-64:v11": EnclaveSpecification(
        proto=AttestationSpecification(
            amdSnp=AttestationSpecificationAmdSnp(
                amdArkDer=amd_snp_ark_der,
                measurement=bytes.fromhex(
                    "4b5c2949eddbe215607a81df61ba75084e705922c76af792d39e437c4ef1f33cc64e04d4bbd40e395bcc3d4bf8daef70"
                ),
                roughtimePubKey=roughtime_public_key,
                authorizedChipIds=amd_snp_authorized_chip_ids,
            )
        ),
        workerProtocols=[1],
        decoder=ContainerWorkerDecoder(),
        clientProtocols=None,
    ),
    "decentriq.r-latex-worker-32-32:v10": EnclaveSpecification(
        proto=AttestationSpecification(
            amdSnp=AttestationSpecificationAmdSnp(
                amdArkDer=amd_snp_ark_der,
                measurement=bytes.fromhex(
                    "5f0a74f6c0633d9d4781d8c112617ce386507b86869b341a89b2183c8e41f01032c3ec9821567530f023f26c0f8a846d"
                ),
                roughtimePubKey=roughtime_public_key,
                authorizedChipIds=amd_snp_authorized_chip_ids,
            )
        ),
        workerProtocols=[1],
        decoder=ContainerWorkerDecoder(),
        clientProtocols=None,
    ),
    "decentriq.r-latex-worker-32-32:v11": EnclaveSpecification(
        proto=AttestationSpecification(
            amdSnp=AttestationSpecificationAmdSnp(
                amdArkDer=amd_snp_ark_der,
                measurement=bytes.fromhex(
                    "de88718b50772045dd4427f0a7b83e80aa5c6a4e33664b88bc37e9663677d724bb3708ede4fca930506f344662141997"
                ),
                roughtimePubKey=roughtime_public_key,
                authorizedChipIds=amd_snp_authorized_chip_ids,
            )
        ),
        workerProtocols=[1],
        decoder=ContainerWorkerDecoder(),
        clientProtocols=None,
    ),
    "decentriq.dataset-sink-worker:v1": EnclaveSpecification(
        proto=AttestationSpecification(
            intelDcap=AttestationSpecificationIntelDcap(
                mrenclave=bytes.fromhex(
                    "3ffc8524bfc85837340ebfaabe4775f498b5040bbd84f30dcf62f2b1f86a61f4"
                ),
                dcapRootCaDer=intel_sgx_dcap_root_ca_der,
                accept_debug=False,
                accept_out_of_date=False,
                accept_configuration_needed=False,
                accept_revoked=False,
            )
        ),
        workerProtocols=[1],
        decoder=DatasetSinkWorkerDecoder(),
        clientProtocols=None,
    ),
    "decentriq.data-source-s3-worker:v1": EnclaveSpecification(
        proto=AttestationSpecification(
            intelDcap=AttestationSpecificationIntelDcap(
                mrenclave=bytes.fromhex(
                    "67b8fe452b4724d85e81210f163936dc3ad1cf9c8602939a27ebe9afa00673ff"
                ),
                dcapRootCaDer=intel_sgx_dcap_root_ca_der,
                accept_debug=False,
                accept_out_of_date=False,
                accept_configuration_needed=False,
                accept_revoked=False,
            )
        ),
        workerProtocols=[1],
        decoder=DataSourceS3WorkerDecoder(),
        clientProtocols=None,
    ),
}


class EnclaveSpecifications:
    """
    Provider of the available enclave specifications provided by the Decentriq platform.

    Enclave specifications enable you to express which particular enclaves you trust.
    The field containing the measurement (e.g. `mrenclave` in the case of Intel SGX) identifies
    the exact binary that will process your data.
    Users of the Decentriq platform are encouraged to reproduce this value by building the enclave
    binary from audited source code and re-producing the measurement (in the case of Intel SGX,
    this would involve simply hashing the produced executable).

    When connecting to the driver enclave, the configured attestation algorithm will guarantee that the
    enclave you connect to is the one corresponding to the enclave specification you chose.
    The associated root certificate will be used to verify that the attestation was signed
    by the expected party (e.g. Intel/AMD/Amazon, depending on the CC technology used).

    Any communication between the driver enclave and worker enclaves handling your data will also
    first be secured by additional attestation procedures. Which enclaves are trusted by the
    driver enclave is controlled by choosing the additional enclave specs from the respective
    compute packages.

    A list of enclave specifications, each encoding your trust in a particular enclave type, can
    be obtained by selecting a subset of the enclave specifications provided by the object
    `decentriq_platform.enclave_specifications`. Selecting the subset of versions should be done
    by calling its `versions` method.
    """

    def __init__(self, specifications: Dict[str, EnclaveSpecification]):
        self.specifications = specifications

    def list(self) -> List[str]:
        """Get a list of all available enclave identifiers."""
        return sorted(self.specifications.keys())

    def versions(self, enclave_versions: List[str]) -> Dict[str, EnclaveSpecification]:
        """
        Get the enclave specifications for the given versioned enclave types.

        Make sure to always include the specification of a *driver enclave*, e.g.
        `"decentriq.driver:v1"` as this is the node with which you communicate directly.
        Add additional versioned enclaves depending on the compute module you use.
        Refer to the main documentation page of each compute module to learn which
        enclaves are available.
        """
        selected_specifications = {}
        for version in enclave_versions:
            enclave_type = version.split(":")[0]
            selected_specifications[enclave_type] = self.specifications[version]
        return selected_specifications

    def all(self) -> List[EnclaveSpecification]:
        """Get a list of all available enclave specifications."""
        return list(self.specifications.values())

    def merge(self, other):
        """
        Merge two sets of enclave specifications into a single set.
        """
        return EnclaveSpecifications({**self.specifications, **other.specifications})


enclave_specifications: EnclaveSpecifications = EnclaveSpecifications(SPECIFICATIONS)
"""The main catalogue of enclave specifications available within the Decentriq platform."""



import os

def _use_tls():
    return False if os.getenv("DECENTRIQ_USE_TLS", "true") == "false" else True

DECENTRIQ_CLIENT_ID = os.getenv("DECENTRIQ_CLIENT_ID", "MHyVW112w7Ql95G96fn9rnLWkYuOLmdk")
DECENTRIQ_HOST = os.getenv("DECENTRIQ_HOST", "api.decentriq.com")
DECENTRIQ_PORT = int(os.getenv("DECENTRIQ_PORT", "443"))
DECENTRIQ_USE_TLS = _use_tls()
DECENTRIQ_REQUEST_RETRY_TOTAL = int(os.getenv("DECENTRIQ_REQUEST_RETRY_TOTAL", "3"))
DECENTRIQ_REQUEST_RETRY_BACKOFF_FACTOR = float(os.getenv("DECENTRIQ_REQUEST_RETRY_BACKOFF_FACTOR", "0"))



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class DatasetSinkWorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INPUTDEPENDENCY_FIELD_NUMBER: builtins.int
    ENCRYPTIONKEYDEPENDENCY_FIELD_NUMBER: builtins.int
    DATASETNAME_FIELD_NUMBER: builtins.int
    DATASETSCOPEID_FIELD_NUMBER: builtins.int
    DATASETDESCRIPTION_FIELD_NUMBER: builtins.int
    DATASETIMPORTID_FIELD_NUMBER: builtins.int
    inputDependency: builtins.str
    encryptionKeyDependency: builtins.str
    datasetName: builtins.str
    datasetScopeId: builtins.str
    datasetDescription: builtins.str
    datasetImportId: builtins.str
    def __init__(
        self,
        *,
        inputDependency: builtins.str = ...,
        encryptionKeyDependency: builtins.str = ...,
        datasetName: builtins.str = ...,
        datasetScopeId: builtins.str = ...,
        datasetDescription: builtins.str | None = ...,
        datasetImportId: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_datasetDescription", b"_datasetDescription", "_datasetImportId", b"_datasetImportId", "datasetDescription", b"datasetDescription", "datasetImportId", b"datasetImportId"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_datasetDescription", b"_datasetDescription", "_datasetImportId", b"_datasetImportId", "datasetDescription", b"datasetDescription", "datasetImportId", b"datasetImportId", "datasetName", b"datasetName", "datasetScopeId", b"datasetScopeId", "encryptionKeyDependency", b"encryptionKeyDependency", "inputDependency", b"inputDependency"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_datasetDescription", b"_datasetDescription"]) -> typing_extensions.Literal["datasetDescription"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_datasetImportId", b"_datasetImportId"]) -> typing_extensions.Literal["datasetImportId"] | None: ...

global___DatasetSinkWorkerConfiguration = DatasetSinkWorkerConfiguration



"""
.. include:: ../../decentriq_platform_docs/gcg_getting_started.md
___
"""
__docformat__ = "restructuredtext"
__pdoc__ = {
    "api": False,
    "attestation": True,
    "authentication": True,
    "builders": False,
    "certs": False,
    "client": False,
    "compute": False,
    "config": False,
    "container": True,
    "graphql": False,
    "helpers": False,
    "node": True,
    "permission": False,
    "proto": False,
    "s3_sink": True,
    "data_source_s3": True,
    "dataset_sink": True,
    "session": True,
    "sql": True,
    "storage": True,
    "types": True,
    "verification": False,
}

from .client import Client, create_client, Session
from .builders import (
    DataRoomBuilder,
    DataRoomCommitBuilder,
    GovernanceProtocol
)
from .compute import Noop, StaticContent
from .permission import Permissions
from .storage import Key
from .attestation import enclave_specifications, EnclaveSpecifications

from .endorsement import Endorser
from .keychain import Keychain

__all__ = [
    "create_client",
    "Client",
    "Session",
    "DataRoomBuilder",
    "DataRoomCommitBuilder",
    "Permissions",
    "GovernanceProtocol",
    "enclave_specifications",
    "EnclaveSpecifications",
    "Key",
    "StaticContent",
    "Noop",
    "sql",
    "container",
    "s3_sink",
    "data_source_s3",
    "dataset_sink",
    "storage",
    "attestation",
    "types",
    "authentication",
    "session",
    "node",
    "Endorser",
    "Keychain",
]
